@@ -1,8 +1,10 @@
-# pylint: disable= [no-self-use, missing-class-docstring]
+# pylint: disable=missing-class-docstring,no-self-use
+from __future__ import annotations
 
 import unittest
 
 import claripy
+import claripy.backends
 
 
 class TestExpression(unittest.TestCase):
@@ -114,15 +116,15 @@ class TestExpression(unittest.TestCase):
         ooo = claripy.BVV(0, 32)
 
         old_formula = claripy.If((old + 1) % 256 == 0, old + 10, old + 20)
-        new_formula = old_formula.replace(old, new)
-        ooo_formula = new_formula.replace(new, ooo)
+        new_formula = claripy.replace(old_formula, old, new)
+        ooo_formula = claripy.replace(new_formula, new, ooo)
 
-        self.assertNotEqual(hash(old_formula), hash(new_formula))
-        self.assertNotEqual(hash(old_formula), hash(ooo_formula))
-        self.assertNotEqual(hash(new_formula), hash(ooo_formula))
+        self.assertNotEqual(old_formula.hash(), new_formula.hash())
+        self.assertNotEqual(old_formula.hash(), ooo_formula.hash())
+        self.assertNotEqual(new_formula.hash(), ooo_formula.hash())
 
-        self.assertEqual(old_formula.variables, {"old"})
-        self.assertEqual(new_formula.variables, {"new"})
+        self.assertEqual(old_formula.variables, frozenset(("old",)))
+        self.assertEqual(new_formula.variables, frozenset(("new",)))
         self.assertEqual(ooo_formula.variables, ooo.variables)
 
         self.assertTrue(old_formula.symbolic)
@@ -137,7 +139,7 @@ class TestExpression(unittest.TestCase):
         new = claripy.BVS("new", 32, explicit_name=True)
         c = (old + 10) - (old + 20)
         d = (old + 1) - (old + 2)
-        cr = c.replace_dict({(old + 10).cache_key: (old + 1), (old + 20).cache_key: (old + 2)})
+        cr = claripy.replace_dict(c, {(old + 10).hash(): (old + 1), (old + 20).hash(): (old + 2)})
         self.assertIs(cr, d)
 
         # test AST collapse
@@ -178,7 +180,7 @@ class TestExpression(unittest.TestCase):
 
     def test_cardinality(self):
         x = claripy.BVS("x", 32)
-        y = claripy.BVS("y", 32, min=100, max=120)
+        y = claripy.BVS("y", 32).annotate(claripy.annotation.StridedIntervalAnnotation(1, 100, 120))
         n = claripy.BVV(10, 32)
         m = claripy.BVV(20, 32)
 
@@ -209,7 +211,6 @@ class TestExpression(unittest.TestCase):
 
     def test_if_stuff(self):
         x = claripy.BVS("x", 32)
-        # y = claripy.BVS('y', 32)
 
         c = claripy.If(x > 10, (claripy.If(x > 10, x * 3, x * 2)), x * 4) + 2
         cc = claripy.If(x > 10, x * 3, x * 4) + 2
@@ -217,17 +218,17 @@ class TestExpression(unittest.TestCase):
         cccc = x * claripy.If(x > 10, claripy.BVV(3, 32), claripy.BVV(4, 32)) + 2
 
         self.assertIs(c, cc)
-        self.assertIs(c.ite_excavated, ccc)
-        self.assertIs(ccc.ite_burrowed, cccc)
+        self.assertIs(claripy.excavate_ite(c), ccc)
+        self.assertIs(claripy.burrow_ite(ccc), cccc)
 
         i = c + c
         ii = claripy.If(x > 10, (x * 3 + 2) + (x * 3 + 2), (x * 4 + 2) + (x * 4 + 2))
-        self.assertIs(i.ite_excavated, ii)
+        self.assertIs(claripy.excavate_ite(i), ii)
 
         cn = claripy.If(x <= 10, claripy.BVV(0x10, 32), 0x20)
         iii = c + cn
         iiii = claripy.If(x > 10, (x * 3 + 2) + 0x20, (x * 4 + 2) + 0x10)
-        self.assertIs(iii.ite_excavated, iiii)
+        self.assertIs(claripy.excavate_ite(iii), iiii)
 
     def test_ite_Solver(self):
         self.raw_ite(claripy.Solver)
@@ -282,7 +283,7 @@ class TestExpression(unittest.TestCase):
         ss = s.branch()
         ss.add(z == itz)
         ss.add(itz != 0)
-        self.assertEqual(ss.eval(y / x, 100), (2,))
+        self.assertEqual(ss.eval(y // x, 100), (2,))
         self.assertEqual(sorted(ss.eval(x, 100)), [1, 10, 100])
         self.assertEqual(sorted(ss.eval(y, 100)), [2, 20, 200])
 
@@ -343,17 +344,13 @@ class TestExpression(unittest.TestCase):
         assert claripy.is_true(b_concat == a100)
 
     def test_true_false_cache(self):
-        claripy.backends._quick_backends.append(claripy.backends.z3)
-
         a = claripy.BVS("a_WILL_BE_VIOLATED", 32)
         c = a == a + 1
-        assert claripy.is_false(c)
+        assert claripy.backends.z3.is_false(c)
         c.args[1].args = (a, claripy.BVV(0, 32))
-        assert claripy.is_false(c)
-        assert not claripy.is_true(c)
-        assert not claripy.is_false(a == a)
-
-        claripy.backends._quick_backends[-1:] = []
+        assert claripy.backends.z3.is_false(c)
+        assert not claripy.backends.z3.is_true(c)
+        assert not claripy.backends.z3.is_false(a == a)
 
     def test_depth_repr(self):
         x = claripy.BVS("x", 32)
@@ -369,11 +366,6 @@ class TestExpression(unittest.TestCase):
             "<BV32 LShR(LShR(LShR(LShR(LShR(<...>, <...>), 0xa), 0xa), 0xa), 0xa)>",
         )
 
-    def test_rename(self):
-        x1 = claripy.BVS("x", 32)
-        x2 = x1._rename("y")
-        assert x2.variables == frozenset(("y",))
-
     def test_canonical(self):
         x1 = claripy.BVS("x", 32)
         b1 = claripy.BoolS("b")
@@ -390,8 +382,8 @@ class TestExpression(unittest.TestCase):
         one_names = frozenset.union(x1.variables, b1.variables, c1.variables)
         two_names = frozenset.union(x2.variables, b2.variables, c2.variables)
 
-        assert frozenset.union(*[a.variables for a in y1.recursive_leaf_asts]) == one_names
-        assert frozenset.union(*[a.variables for a in y2.recursive_leaf_asts]) == two_names
+        assert frozenset.union(*[a.variables for a in y1.leaf_asts()]) == one_names
+        assert frozenset.union(*[a.variables for a in y2.leaf_asts()]) == two_names
         assert y1.canonicalize()[-1] is y2.canonicalize()[-1]
 
     def test_depth(self):
@@ -426,19 +418,16 @@ class TestExpression(unittest.TestCase):
         assert len(x_xor.args) == 4
         assert len(x_and.args) == 4
 
-        assert (x_add).replace(x, o).args[0] == 8
-        assert (x_mul).replace(x, o).args[0] == 16
-        assert (x_or).replace(x, o).args[0] == 7
-        assert (x_xor).replace(x, o).args[0] == 0
-        assert (x_and).replace(x, o).args[0] == 0
-        assert (100 + (x_sub).replace(x, o)).args[0] == 90
+        assert claripy.replace(x_add, x, o).args[0] == 8
+        assert claripy.replace(x_mul, x, o).args[0] == 16
+        assert claripy.replace(x_or, x, o).args[0] == 7
+        assert claripy.replace(x_xor, x, o).args[0] == 0
+        assert claripy.replace(x_and, x, o).args[0] == 0
+        assert (100 + claripy.replace(x_sub, x, o)).args[0] == 90
 
-        # make sure that all backends handle this properly
-        for b in claripy.backends._all_backends:
-            try:
-                b.convert(x + x + x + x)
-            except claripy.BackendError:
-                pass
+        # make sure that z3 and vsa backends handle this properly
+        claripy.backends.z3.convert(x + x + x + x)
+        claripy.backends.vsa.convert(x + x + x + x)
 
     def test_signed_concrete(self):
         bc = claripy.backends.concrete
@@ -448,10 +437,10 @@ class TestExpression(unittest.TestCase):
         d = claripy.BVV(-3, 32)
 
         # test unsigned
-        assert bc.convert(a / c) == 1
-        assert bc.convert(a / d) == 0
-        assert bc.convert(b / c) == 0x55555553
-        assert bc.convert(b / d) == 0
+        assert bc.convert(a // c) == 1
+        assert bc.convert(a // d) == 0
+        assert bc.convert(b // c) == 0x55555553
+        assert bc.convert(b // d) == 0
         assert bc.convert(a % c) == 2
         assert bc.convert(a % d) == 5
         assert bc.convert(b % c) == 2
@@ -479,10 +468,10 @@ class TestExpression(unittest.TestCase):
         solver.add(d == -3)
 
         # test unsigned
-        assert list(solver.eval(a / c, 2)) == [1]
-        assert list(solver.eval(a / d, 2)) == [0]
-        assert list(solver.eval(b / c, 2)) == [0x55555553]
-        assert list(solver.eval(b / d, 2)) == [0]
+        assert list(solver.eval(a // c, 2)) == [1]
+        assert list(solver.eval(a // d, 2)) == [0]
+        assert list(solver.eval(b // c, 2)) == [0x55555553]
+        assert list(solver.eval(b // d, 2)) == [0]
         assert list(solver.eval(a % c, 2)) == [2]
         assert list(solver.eval(a % d, 2)) == [5]
         assert list(solver.eval(b % c, 2)) == [2]
@@ -563,6 +552,15 @@ class TestExpression(unittest.TestCase):
         except claripy.ClaripyOperationError:
             pass
 
+    def test_bool_replace_in_ite(self):
+        b = claripy.BoolS("b")
+        expr = claripy.If(b, claripy.BVV(2, 32), claripy.BVV(3, 32))
+        new_expr = claripy.replace(expr, b, claripy.BoolV(True))
+
+        # Replace calls make_like which will simplify the expression. As a
+        # result, the new expression will be a BVV.
+        assert new_expr.op == "BVV" and new_expr.args == (2, 32)
+
 
 if __name__ == "__main__":
     unittest.main()
