@@ -1,19 +1,19 @@
-from common_backend_smt_solver import if_installed
+# pylint: disable=missing-class-docstring,no-self-use
+from __future__ import annotations
+
 from unittest import TestCase, main
+
 import claripy
-
-import logging
-
-l = logging.getLogger("claripy.test.solver")
+import claripy.frontend
 
 
 def raw_hybrid_solver(reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     s = claripy.SolverHybrid()
 
-    x = claripy.BVS("x", 32, min=0, max=10, stride=2)
-    y = claripy.BVS("y", 32, min=20, max=30, stride=5)
+    x = claripy.BVS("x", 32).annotate(claripy.annotation.StridedIntervalAnnotation(2, 0, 10))
+    y = claripy.BVS("y", 32).annotate(claripy.annotation.StridedIntervalAnnotation(5, 20, 30))
 
     # TODO: for now, the stride isn't respected in symbolic mode, but we'll fix that next.
     # until we do, let's add constraints
@@ -31,10 +31,10 @@ def raw_hybrid_solver(reuse_z3_solver):
 
     # test approximate_first option
     s._approximate_first = True
-    old_solve_count = claripy._backends_module.backend_z3.solve_count
+    old_solve_count = claripy.backends.z3.solve_count
     assert sorted(s.eval(x, 20)) == [0, 2, 4, 6, 8, 10]
-    assert claripy._backends_module.backend_z3.solve_count == old_solve_count
-    s._approximate_firsrt = False
+    assert claripy.backends.z3.solve_count == old_solve_count
+    s._approximate_first = False
 
     # now constrain things further so that the VSA overapproximates
     s.add(x <= 4)
@@ -48,7 +48,6 @@ def raw_hybrid_solver(reuse_z3_solver):
     t = claripy.SolverHybrid()
     x = claripy.BVS("x", 32)
     t.add(x <= 10)
-    print(t.eval(x, 80, exact=False))
     assert len(t.eval(x, 5, exact=False)) == 5
     assert len(t.eval(x, 5, exact=False)) == 5
     assert len(t.eval(x, 6, exact=False)) == 6
@@ -57,7 +56,7 @@ def raw_hybrid_solver(reuse_z3_solver):
 
 
 def raw_replacement_solver(reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     sr = claripy.SolverReplacement()
     x = claripy.BVS("x", 32)
@@ -67,19 +66,19 @@ def raw_replacement_solver(reuse_z3_solver):
 
     y = claripy.BVS("y", 32)
     sr.add([y + 1 == 200])
-    assert (y + 1).cache_key in sr._replacements
+    assert (y + 1).hash() in sr._replacements
     assert sr._replacement(y + 1) is claripy.BVV(200, 32)
 
     srb = sr.branch()
     assert len(srb.constraints) == len(sr.constraints)  # pylint:disable=no-member
-    assert (y + 1).cache_key in sr._replacements
+    assert (y + 1).hash() in sr._replacements
     assert sr._replacement(y + 1) is claripy.BVV(200, 32)
 
     sr = claripy.SolverReplacement()
     b = claripy.BoolS("b")
     assert sr._replacement(b) is b
     sr.add(claripy.Not(b))
-    assert sr._replacement(b) is claripy.false
+    assert sr._replacement(b) is claripy.false()
 
     sr = claripy.SolverReplacement(claripy.SolverVSA(), complex_auto_replace=True)
     x = claripy.BVS("x", 64)
@@ -89,11 +88,7 @@ def raw_replacement_solver(reuse_z3_solver):
 
 
 def raw_solver(solver_type, reuse_z3_solver):
-    # bc = claripy.backends.BackendConcrete(clrp)
-    # bz = claripy.backends.BackendZ3(clrp)
-    # claripy.expression_backends = [ bc, bz, ba ]
-
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     s = solver_type()
 
@@ -103,8 +98,6 @@ def raw_solver(solver_type, reuse_z3_solver):
     y = claripy.BVS("y", 32)
     z = claripy.BVS("z", 32)
 
-    l.debug("adding constraints")
-
     s.add(x == 10)
     s.add(y == 15)
 
@@ -115,7 +108,6 @@ def raw_solver(solver_type, reuse_z3_solver):
     assert results[0][1] == 16
     assert results[0][2] == 3
 
-    l.debug("checking")
     assert s.satisfiable()
     assert not s.satisfiable(extra_constraints=[x == 5])
     assert s.eval(x + 5, 1)[0] == 15
@@ -128,16 +120,16 @@ def raw_solver(solver_type, reuse_z3_solver):
     assert len(shards) == 2
     assert len(shards[0].variables) == 1
     assert len(shards[1].variables) == 1
-    if isinstance(s, claripy.frontend_mixins.ConstraintExpansionMixin) or (
-        isinstance(s, claripy.frontends.HybridFrontend)
-        and isinstance(s._exact_frontend, claripy.frontend_mixins.ConstraintExpansionMixin)
+    if isinstance(s, claripy.frontend.mixin.ConstraintExpansionMixin) or (
+        isinstance(s, claripy.frontend.HybridFrontend)
+        and isinstance(s._exact_frontend, claripy.frontend.mixin.ConstraintExpansionMixin)
     ):  # the hybrid frontend actually uses the exact frontend for the split
         assert {len(shards[0].constraints), len(shards[1].constraints)} == {
             2,
             1,
         }  # adds the != from the solution() check
-    if isinstance(s, claripy.frontends.ReplacementFrontend):
-        assert {len(shards[0].constraints), len(shards[1].constraints)} == {1, 1}
+    if isinstance(s, claripy.frontend.ReplacementFrontend):
+        assert {len(shards[0].constraints), len(shards[1].constraints)} == {1}
 
     # test result caching
     s = solver_type()
@@ -162,36 +154,13 @@ def raw_solver(solver_type, reuse_z3_solver):
     s.add(claripy.ULT(z, 5))
     assert len(s.constraints) == old_count
 
-    # print("========================================================================================")
-    # print("========================================================================================")
-    # print("========================================================================================")
-    # print("========================================================================================")
-    # a = s.eval(z, 100)
-    # print("ANY:", a)
-    # print("========================================================================================")
-    # mx = s.max(z)
-    # print("MAX",mx)
-    # print("========================================================================================")
-    # mn = s.min(z)
-    # print("MIN",mn)
-    # print("========================================================================================")
-    # print("========================================================================================")
-    # print("========================================================================================")
-    # print("========================================================================================")
-
-    print("CONSTRAINT COUNTS:", [len(_.constraints) for _ in s.split()])
-
     assert s.max(z) == 4
     assert s.min(z) == 0
     assert s.min(y) == 22
     assert s.max(y) == 2 ** y.size() - 1
 
-    print("CONSTRAINT COUNTS:", [len(_.constraints) for _ in s.split()])
-
     ss = s.split()
     assert len(ss) == 2
-    # if isinstance(s, claripy.frontend_mixins.ConstraintExpansionMixin):
-    #   assert { len(_.constraints) for _ in ss } == { 3, 2 }
 
     # Batch evaluation
     s.add(y < 24)
@@ -242,31 +211,31 @@ def raw_solver(solver_type, reuse_z3_solver):
 
     # test result caching
 
-    if isinstance(s, claripy.frontend_mixins.ModelCacheMixin):
-        count = claripy._backends_module.backend_z3.solve_count
+    if isinstance(s, claripy.frontend.mixin.ModelCacheMixin):
+        count = claripy.backends.z3.solve_count
 
         s = solver_type()
         x = claripy.BVS("x", 32)
         s.add(x == 10)
         assert s.satisfiable()
-        assert claripy._backends_module.backend_z3.solve_count == count
+        assert claripy.backends.z3.solve_count == count
         assert s.eval(x, 1)[0] == 10
-        assert claripy._backends_module.backend_z3.solve_count == count
+        assert claripy.backends.z3.solve_count == count
         s.add(x == 10)
         s.add(x > 9)
         assert s.eval(x, 1)[0] == 10
-        assert claripy._backends_module.backend_z3.solve_count == count
+        assert claripy.backends.z3.solve_count == count
 
         y = claripy.BVS("y", 32)
         s.add(y < 999)
         assert s.satisfiable()
-        assert claripy._backends_module.backend_z3.solve_count == count
+        assert claripy.backends.z3.solve_count == count
         assert s.eval(y, 1)[0] == 0
-        assert claripy._backends_module.backend_z3.solve_count == count
+        assert claripy.backends.z3.solve_count == count
 
 
 def raw_solver_branching(solver_type, reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     s = solver_type()
     x = claripy.BVS("x", 32)
@@ -277,7 +246,7 @@ def raw_solver_branching(solver_type, reuse_z3_solver):
     assert s.eval(x, 1)[0] > 0
 
     t = s.branch()
-    if isinstance(s, claripy.frontends.FullFrontend):
+    if isinstance(s, claripy.frontend.FullFrontend):
         assert s._tls.solver is t._tls.solver
         assert s._finalized
         assert t._finalized
@@ -298,7 +267,7 @@ def raw_solver_branching(solver_type, reuse_z3_solver):
 
 
 def raw_combine(solver_type, reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     s10 = solver_type()
     s20 = solver_type()
@@ -321,7 +290,7 @@ def raw_combine(solver_type, reuse_z3_solver):
 
 
 def raw_composite_solver(reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     # pylint:disable=no-member
     s = claripy.SolverComposite()
@@ -365,17 +334,17 @@ def raw_composite_solver(reuse_z3_solver):
     c = claripy.And(x == 1, y == 2, z > 3)
     s.add(c)
 
-    if isinstance(s._template_frontend, claripy.frontend_mixins.ModelCacheMixin):
+    if isinstance(s._template_frontend, claripy.frontend.mixin.ModelCacheMixin):
         assert len(s._solver_list) == 3
-        count = claripy._backends_module.backend_z3.solve_count
+        count = claripy.backends.z3.solve_count
         assert s.satisfiable()
-        assert claripy._backends_module.backend_z3.solve_count == count + 3
+        assert claripy.backends.z3.solve_count == count + 3
         assert list(s.eval(x + y, 1)) == [3]
-        assert claripy._backends_module.backend_z3.solve_count == count + 3
+        assert claripy.backends.z3.solve_count == count + 3
 
 
 def raw_minmax(reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     s = claripy.Solver()
     x = claripy.BVS("x", 32)
@@ -386,7 +355,7 @@ def raw_minmax(reuse_z3_solver):
 
 
 def raw_composite_discrepancy(reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     a = claripy.BVS("a", 8)
     b = claripy.BVS("b", 8)
@@ -415,14 +384,12 @@ def raw_composite_discrepancy(reuse_z3_solver):
 
     sn.add(constraints)
     sc.add(constraints)
-    print(sn.max(dst), sc.max(dst))
-    print(sn.min(dst), sc.min(dst))
     assert sn.max(dst) == sc.max(dst)
     assert sn.min(dst) == sc.min(dst)
 
 
 def raw_ancestor_merge(solver, reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     s = solver()
     x = claripy.BVS("x", 32)
@@ -437,10 +404,10 @@ def raw_ancestor_merge(solver, reuse_z3_solver):
     p.add(z == 1)
     q.add(z == 2)
 
-    r = p.merge([q], [claripy.true, claripy.true])[-1]
+    r = p.merge([q], [claripy.true(), claripy.true()])[-1]
     t = p.merge([q], [p.constraints[-1], q.constraints[-1]], common_ancestor=s)[-1]
 
-    if not isinstance(r, claripy.frontends.CompositeFrontend):
+    if not isinstance(r, claripy.frontend.CompositeFrontend):
         assert len(r.constraints) == 1
     assert len(t.constraints) == 3
     assert t.constraints[-1].variables == z.variables
@@ -449,7 +416,7 @@ def raw_ancestor_merge(solver, reuse_z3_solver):
 
 
 def raw_unsat_core(solver, reuse_z3_solver):
-    claripy._backend_z3.reuse_z3_solver = reuse_z3_solver
+    claripy.backends.z3.reuse_z3_solver = reuse_z3_solver
 
     s = solver(track=True)
     x = claripy.BVS("x", 32)
@@ -477,15 +444,12 @@ def raw_unsat_core(solver, reuse_z3_solver):
 
 
 class StandardTests(TestCase):
-    @if_installed
     def test_composite_solver_with_strings(self):
-        s = claripy.SolverComposite(
-            template_solver_string=claripy.SolverCompositeChild(backend=claripy.backend_manager.backends.smtlib_cvc4)
-        )
+        s = claripy.SolverComposite()
         x = claripy.BVS("x", 32)
         y = claripy.BVS("y", 32)
         z = claripy.BVS("z", 32)
-        str_1 = claripy.StringS("sym_str_1", 1024)
+        str_1 = claripy.StringS("sym_str_1")
         c = claripy.And(x == 1, y == 2, z == 3, str_1 == claripy.StringV("cavallo"))
         s.add(c)
         assert len(s._solver_list) == 4
@@ -520,7 +484,7 @@ class StandardTests(TestCase):
         s = claripy.Solver()
         s.add(x == 10)
         assert s.satisfiable()
-        s.add(claripy.false)
+        s.add(claripy.false())
         assert not s.satisfiable()
 
     def test_simplification_annotations(self):
@@ -537,13 +501,13 @@ class StandardTests(TestCase):
 
     def test_zero_division_in_cache_mixin(self):
         # Bug in the caching backend. See issue #49 on github.
-        num = claripy.BVS("num", 256)
-        denum = claripy.BVS("denum", 256)
-        e = claripy.BVS("e", 256)
+        num = claripy.BVS("num", 8)
+        denum = claripy.BVS("denum", 8)
+        e = claripy.BVS("e", 8)
         s = claripy.Solver()
         s.add(e == 8)
         assert s.satisfiable()
-        s.add(claripy.If(denum == 0, 0, num / denum) == e)
+        s.add(claripy.If(denum == 0, 0, num // denum) == e)
         assert s.satisfiable()
         # As a bonus:
         s.add(num == 16)
@@ -587,13 +551,11 @@ class StandardTests(TestCase):
         s = claripy.Solver()
         x = claripy.BVS("x", 32)
         s.add(x >= 19)
-        print(s.min(x, extra_constraints=[x >= 20]))
         assert s.min(x) == 19
 
         s = claripy.Solver()
         x = claripy.BVS("x", 32)
         s.add(x <= 19)
-        print(s.max(x, extra_constraints=[x <= 18]))
         assert s.max(x) == 19
 
     def test_cached_max(self):
@@ -619,6 +581,8 @@ class StandardTests(TestCase):
 
 
 class Base:
+    solver: type[claripy.Solver]
+
     def test_solver_with_reuse(self):
         raw_solver(self.solver, True)
 
@@ -707,19 +671,15 @@ class TestComposite(TestCase, UnsatCore):
         s.add(claripy.If(aa[15:0] >= 0x1, claripy.BVV(0x1, 32), claripy.BVV(0x0, 32)) == 1)
         s.add(0xFFFFFFF + cc + bb == 0)
         s.add(claripy.If((claripy.If(bb == 0x0, bb, dd) | ee) == 0, claripy.BVV(1, 1), claripy.BVV(0, 1)) != 1)
-        # print(s._solvers)
 
         s.add(dd == 0xB)
         s.simplify()
-        # print(s._solvers)
 
         s.add(bb == 0x8004)
         s.simplify()
-        # print(s._solvers)
 
         s.add(ee == 0)
         s.simplify()
-        # print(s._solvers)
 
         bb_values = s.eval(bb, n=8)
         assert len(bb_values) == 1
